﻿/**
 * @ngdoc service
 * @name translatejobService
 * @function
 * 
 * @description
 * Service for getting and setting job things...
 */

(function() {
    'use strict';

    // var serviceRoot = "backoffice/translateplus/translationJobApi/";
  

    function jobService($http, localizationService) {

        var serviceRoot = Umbraco.Sys.ServerVariables.translatePlus.JobService;

        var service = {
            getAll: getAll,
            getById: getById,
            getFullById: getFullById,
            getByCultureId: getByCultureId,
            getByCultureAndStatus: getByCultureAndStatus,
            getArchivedByCulture: getArchivedByCulture,

            getAllNodesInJob: getAllNodesInJob,
            getNodesByJob: getNodesByJob,
            getFullNodesInJob: getFullNodesInJob,

            getSummaryInfo: getSummaryInfo,
            getSummaryRange: getSummaryRange,

            getStatusForContent: getStatusForContent,
            getTargetStatus: getTargetStatus,

            getOpenSiblings: getOpenSiblings,

            create: create,
            submit: submit,
            check: check,
            checkById: checkById,
            approve: approve,
            archiveJob: archiveJob,
            cancel: cancel,
            remove: remove,
            resetStatus: resetStatus,

            getProviders: getProviders,
            getProvider: getProvider,

            getStatusName: getStatusName,

            getNewGroupId: getNewGroupId,
            submitGroup: submitGroup
        };

        return service;

        ////////////

        function getAll() {
            return $http.get(serviceRoot + "GetAll");
        }

        function getById(id) {
            return $http.get(serviceRoot + "Get/" + id);
        }

        function getFullById(id) {
            return $http.get(serviceRoot + "GetFull/" + id);
        }

        function getByCultureId(id) {
            return $http.get(serviceRoot + "GetByCultureId/" + id + "?page=1");
        }

        function getByCultureAndStatus(cultureId, statusMin, statusMax, page) {
            return $http.get(serviceRoot + "GetByCultureAndStatus/" + 
                        cultureId + "?min=" + statusMin + "&max=" + statusMax + "&page=" + page);
        }

        function getArchivedByCulture(cultureId, page) {
            return $http.get(serviceRoot + "GetArchivedByCulture/" + cultureId + "?page=" + page);
        }

        function getSummaryInfo(status) {
            return $http.get(serviceRoot + "GetSummaryInfo?status=" + status);
        }

        function getSummaryRange(minStatus, maxStatus) {
            return $http.get(serviceRoot + "GetSummaryRange?min=" + minStatus + "&max=" + maxStatus);
        }

        function getAllNodesInJob(jobId) {
            return $http.get(serviceRoot + "GetAllNodesInJob/" + jobId);
        }

        function getFullNodesInJob(jobId) {
            return $http.get(serviceRoot + "GetFullNodesInJob/" + jobId);
        }

        function getNodesByJob(jobId, page) {
            return $http.get(serviceRoot + "GetNodesByJob/" + jobId + "?page=" + page);
        }

        function getStatusForContent(contentId) {
            return $http.get(serviceRoot + "GetStatusForContent/" + contentId)
        }

        function getTargetStatus(contentId) {
            return $http.get(serviceRoot + "GetStatusForTarget/" + contentId);
        }


        function getOpenSiblings(id) {
            return $http.get(serviceRoot + "GetOpenSiblings/" + id);
        }

        function getNewGroupId() {
            return $http.get(serviceRoot + "GetNewGroupId");
        }

        function submitGroup(providerKey, groupId) {
            return $http.post(serviceRoot + "SubmitGroup/?providerKey=" + providerKey + "&groupId=" + groupId);
        }

        function create(name, nodes, providerKey, providerOptions, groupId) {

            var createOptions = {
                Name : name,
                ProviderKey : providerKey,
                Nodes: nodes,
                ProviderOptions: providerOptions,
                GroupId: groupId
            };


            return $http.post(serviceRoot + "Create", createOptions);
        }

        function submit(id) {
            return $http.post(serviceRoot + "Submit/" + id);
        }

        function check() {
            return $http.get(serviceRoot + "GetPending");
        }

        function checkById(id) {
            return $http.get(serviceRoot + "GetPendingById/" + id);
        }

        function approve(id, nodes, publish, approve, check) {

            var approveOptions = {
                NodeIds: nodes, 
                Publish: publish,
                Approve: approve,
                Check: check
            };

            return $http.post(serviceRoot + "Approve/" + id, approveOptions);
        }

        function archiveJob(id) {
            return $http.post(serviceRoot + "ArchiveJob/" + id);
        }

        function cancel(id) {
            return $http.post(serviceRoot + "Cancel/" + id);
        }

        function remove(id) {
            return $http.post(serviceRoot + "Remove/" + id);
        }

        function removeAll(id) {
            return $http.post(serviceRoot + "RemoveAll/" + id);
        }

        function resetStatus(id, status) {
            return $http.post(serviceRoot + "ResetJob/" + id + "?status=" + status);
        }

        function getProviders () {
            return $http.get(serviceRoot + "GetProviders");
        }

        function getProvider(key) {
            return $http.get(serviceRoot + "GetProvider/" + key);
        }

        function getStatusName(jobStatus) {
            // TODO: Locallize the status name.
            // return jobStatus;
            // return localizationService.localize("translateJobStatus_" + jobStatus);
            return jobStatus;
        }

    }

    angular.module('umbraco.resources')
        .factory('translateJobService', jobService);
})();

